package cz.muni.fi.pb162.runtimeexception;

/**
 * Základnú teóriu ohľadom výnimiek nájdete v balíku exception v triede Demo
 */
public class Demo {
    public static void main(String[] args) {
        // Behové výnimky reprezentujú chyby v programe
        // Sú potomkami triedy RuntimeException
        // Nie sú sledované prekladačom
        // Príklad vlastnej behovej výnimky je trieda MissingNameException
        // Príklad použitia výnimiek nájdete v triede Person v konštruktore

        // príklad zachytenej behovej výnimky
        creatingPersonCatchException();

        // v metóde nájdete ukážku použitia bloku finally
        // táto metóda vyhodí výnimku a ukončí program
        creatingPersonWithFinallyBlock();
    }

    private static void creatingPersonCatchException() {
        try {
            System.out.println("Snažím sa vytvoriť osobu s hodnotou null ako menom!");
            Person peter = new Person(null);
            System.out.println("Toto sa nevypíše!");
        } catch (MissingNameException ex) {
            System.out.println("Bola vyhodená výnimka MissingNameException. Chyba: " + ex.getMessage());
        } catch (IllegalArgumentException ex) {
            System.out.println("Bola vyhodená výnimka IllegalArgumentException. Chyba: " + ex.getMessage());
        }

        try {
            System.out.println("Snažím sa vytvoriť osobu s prázdnym Stringom ako menom!");
            Person peter = new Person("");
            System.out.println("Toto sa nevypíše!");
        // ak sa telo catch bloku opakuje môžeme využiť takýto skrátený zápis
        } catch (MissingNameException | IllegalArgumentException ex) {
            System.out.println("Použili sme nevhodnú hodnotu pre meno osoby! Chyba: " + ex.getMessage());
        }
    }

    // blok finally obvykle nasleduje catch bloky (môže byť aj priamo za try blokom)
    // kód v bloku finally sa vykoná vždy (ak je výnimka zachytená, nezachytená alebo ak nebola vyhodená)
    private static void creatingPersonWithFinallyBlock() {
        try {
            System.out.println("Snažím sa vytvoriť osobu s hodnotou null ako menom!");
            Person peter = new Person(null);
            System.out.println("Toto sa nevypíše!");
        } catch (MissingNameException ex) {
            System.out.println("Bola vyhodená výnimka MissingNameException. Chyba: " + ex.getMessage());
        } finally { // blok finally sa vykoná aj keď výnimka bola zachytená
            System.out.println("Blok finally sa vykoná vždy!");
        }

        try {
            System.out.println("Snažím sa vytvoriť osobu s prázdnym Stringom ako menom!");
            Person peter = new Person("");
            System.out.println("Toto sa nevypíše!");
        } finally { // blok finally sa vykoná aj keď výnimka nebola zachytená
            System.out.println("Blok finally sa vykoná vždy aj keď sa výnimka nezachytí!");
        }
    }
}
